package tools.deployment;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipOutputStream;

import org.netkernel.container.IKernel;
import org.netkernel.container.config.IConfiguration;
import org.netkernel.layer0.boot.IModule;
import org.netkernel.layer0.boot.ModuleManager;
import org.netkernel.layer0.boot.ModuleSourceWithLevel;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.representation.impl.HDSPredicateFactory;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.xml.xda.DOMXDA;
import org.netkernel.xml.xda.IXDA;
import org.netkernel.xml.xda.IXDAIterator;
import org.netkernel.xml.xda.IXDAReadOnly;
import org.netkernel.xml.xda.IXDAReadOnlyIterator;

import com.sun.org.apache.xerces.internal.parsers.IntegratedParserConfiguration;

public class DeploymentEditorAccessor extends StandardAccessorImpl
{
	public DeploymentEditorAccessor()
	{	declareThreadSafe();
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		IHDSNode param = (IHDSNode)aContext.source("httpRequest:/params",IHDSNode.class);
		//System.out.println(param);
		String action=(String)param.getFirstValue("action2");
		if ("edit".equals(action))
		{	onList(aContext,param);
		}
		if ("update".equals(action))
		{	onUpdate(aContext,param);
		}
		else
		{	onList(aContext,param);
		}
	}
	
	private void onList(INKFRequestContext aContext, IHDSNode param) throws Exception
	{
		HDSBuilder b2=new HDSBuilder();
		if (param.getChildren().length==0)
		{	b2.addNode("sortby","runlevel");
		}
		else
		{	b2.importChildren(param);
		}
		b2.addNode("value", "2");
		param=b2.getRoot();
		
		INKFRequest req=aContext.createRequest("active:xslt");
		req.addArgument("operator", "res:/tools/deployment/styleDeployment.xsl");
		req.addArgument("operand", "netkernel:/deployment");
		req.setRepresentationClass(aContext.getThisRequest().getRepresentationClass());
		req.addArgumentByValue("param",param);
		INKFResponseReadOnly resp=aContext.issueRequestForResponse(req);
		INKFResponse resp2=aContext.createResponseFrom( resp );
	    resp2.setHeader("WrappedControlPanel", true);
		
	}
	
	private static IModule getModuleWithSource(URI aSrc, List<IModule> aModules)
	{	IModule result=null;
		for (IModule m: aModules)
		{	if (m.getSource().equals(aSrc))
			{	result=m;
				break;
			}
		}
		return result;
	}
	
	private void onUpdate(INKFRequestContext aContext, IHDSNode param) throws Exception
	{
		String src=(String)param.getFirstValue("src");
		IHDSNode deployment = aContext.source("netkernel:/deployment",IHDSNode.class);
		IHDSNode module = deployment.getNodes("/modules/module").filter(HDSPredicateFactory.namedChildStringEquals("src", src)).getFirstNode();
		
		int requiredRunlevel = -1;
		try
		{	requiredRunlevel=Integer.parseInt((String)param.getFirstValue("runlevel"));
		}
		catch (Exception e) {;}
		int currentRunlevel = -1;
		try
		{	currentRunlevel = Integer.parseInt((String)module.getFirstValue("runlevel"));
		}
		catch (Exception e) {;}
		
		boolean requiredExpansion = param.getFirstValue("expanded")!=null;
		boolean currentExpansion = !((Boolean)module.getFirstValue("jar"));
		
		boolean requiredEnablement = param.getFirstValue("enabled")!=null;
		boolean currentEnablement = ((Boolean)module.getFirstValue("enabled"));
		
		HDSBuilder b = new HDSBuilder();
		b.pushNode("update");
		
		String oldSrc=src;
		if (requiredExpansion && !currentExpansion)
		{	src=doExpand(src);
			b.addNode("expand", src);
		}
		if (!requiredExpansion && currentExpansion)
		{	src=doJar(src);
			b.addNode("jar", src);
		}

		if (requiredRunlevel!=currentRunlevel)
		{	b.addNode("runlevel", requiredRunlevel);
		}
		if (requiredEnablement!=currentEnablement)
		{	b.addNode("enable", requiredEnablement);
		}
		boolean needsUpdate = requiredRunlevel!=currentRunlevel || requiredExpansion!=currentExpansion || requiredEnablement!=currentEnablement;
		b.addNode("updated",needsUpdate);
		
		INKFRequest req=aContext.createRequest("active:xslt");
		req.addArgument("operator", "res:/tools/deployment/styleDeployment.xsl");
		req.addArgumentByValue("operand", b.getRoot());
		req.setRepresentationClass(aContext.getThisRequest().getRepresentationClass());
		INKFResponseReadOnly resp=aContext.issueRequestForResponse(req);
		INKFResponse resp2=aContext.createResponseFrom( resp );
	    resp2.setHeader("WrappedControlPanel", true);
		
		if (needsUpdate)
		{
			IConfiguration config=aContext.getKernelContext().getKernel().getConfiguration();
			String installPathS=config.getString("netkernel.install.path");
			String initModulesS=config.getString("netkernel.init.modules");
			URI installPath=URI.create(installPathS);
			URI initModules=installPath.resolve(initModulesS);
			IXDA moduleXML = aContext.source(initModules.toString(),IXDAReadOnly.class).getClone();
			
			IXDAIterator m=moduleXML.iterator("/modules/*");
			while (m.hasNext())
			{	m.next();
				String moduleSource = m.getText(".", true);
				String absoluteSource = installPath.resolve(moduleSource).toString();
				if (absoluteSource.equals(oldSrc))
				{	
					DOMXDA replacement = new DOMXDA(XMLUtils.newDocument(),false);
					String name=requiredEnablement?"module":"disabled";
					replacement.appendPath("/", name, src);
					if (requiredRunlevel>0)
					{	replacement.appendPath("/"+name, "@runlevel", Integer.toString(requiredRunlevel));
					}
					
					moduleXML.replace(replacement, "/", m.getCurrentXPath());
					aContext.sink(initModules.toString(), moduleXML);
					break;
				}
			}
			
			
		}
		
		
	}
	
	private String doJar(String aSrc) throws Exception
	{	File src=new File(new URI(aSrc));
		String destURI = aSrc.substring(0,aSrc.length()-1)+".jar";
		File dest = new File(new URI(destURI));
		try
		{	ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(dest));
			ZipUtils.zipDir(src, zos);
			zos.close();
			return destURI;
		}
		catch (Exception e)
		{	dest.delete();
			throw e;
		}
	}
	
	private String doExpand(String aSrc) throws Exception
	{	
		File jarFile = new File(new URI(aSrc));
		int i=aSrc.lastIndexOf(".");
		String dest;
		if (i>0)
		{	dest=aSrc.substring(0,i)+"/";
		}
		else
		{	dest=aSrc+"-expanded/";
		}
		File destFile = new File(new URI(dest));
		try
		{	FileInputStream in = new FileInputStream(jarFile);
			ZipUtils.unzip(in, destFile);
			return dest;
		}
		catch (Exception e)
		{	NewModuleWizardAccessor.cleanupDir(destFile);
			throw e;
		}
		
	}

	
}
