package tools.deployment;

import java.io.File;
import java.net.URI;
import java.util.List;

import org.netkernel.container.config.IConfiguration;
import org.netkernel.layer0.boot.IModule;
import org.netkernel.layer0.boot.ModuleManager;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.urii.impl.Version;
import org.netkernel.xml.xda.DOMXDA;
import org.netkernel.xml.xda.IXDA;
import org.netkernel.xml.xda.IXDAReadOnly;
import org.netkernel.xml.xda.XPathLocationException;
import org.w3c.dom.Node;

public class NewModuleWizardAccessor extends StandardAccessorImpl
{
	public NewModuleWizardAccessor()
	{	declareThreadSafe();
	}

	public void onSource(INKFRequestContext aContext) throws Exception
	{
		IHDSNode param = (IHDSNode)aContext.source("httpRequest:/params",IHDSNode.class);
		//System.out.println(param);
		if (param.getChildren().length==0)
		{	aContext.delete("session:/newModuleWizardState");
		}
		
		HDSBuilder wizardState = new HDSBuilder();
		try
		{	IHDSNode wizardStateIn= aContext.source("session:/newModuleWizardState",IHDSNode.class);
			wizardState.importChildren(wizardStateIn);
		}
		catch (Exception e)
		{	// initialise default values
			wizardState.addNode("page", "1");
			wizardState.addNode("name", "");
			wizardState.addNode("desc", "Created with New Module Wizard");
			wizardState.addNode("urn", "urn:");
			wizardState.addNode("version", "1.0.0");
			wizardState.addNode("source", "");
			wizardState.addNode("fulcrum", "");
			wizardState.addNode("internal", "/resources/");
			wizardState.addNode("external", "");
			wizardState.addNode("doc", "on");
			wizardState.addNode("test", "on");
			wizardState.addNode("example", "");
		}
		
		IHDSNode wizardStateRead=wizardState.getRoot();

		// copy parameter values into state
		for (IHDSNode n: param.getNodes("*"))
		{	String name=n.getName();
			IHDSNode n2=wizardStateRead.getFirstNode(name);
			if (n2!=null)
			{	wizardState.setCursor(n2);
				wizardState.setValue(n.getValue());
			}
		}
		String pageNumber = (String)wizardStateRead.getFirstValue("page");
		//special case(!) for checkboxes
		if (pageNumber.equals("3"))
		{	if (param.getFirstNode("doc")==null)
			{	IHDSNode n2=wizardStateRead.getFirstNode("doc");
				wizardState.setCursor(n2);
				wizardState.setValue("off");
			}
			if (param.getFirstNode("test")==null)
			{	IHDSNode n2=wizardStateRead.getFirstNode("test");
				wizardState.setCursor(n2);
				wizardState.setValue("off");
			}
		}
		
		boolean submit=param.getChildren().length>1;
		HDSBuilder validation = new HDSBuilder();
		validation.pushNode("v");
		
		if (submit)
		{	//validate
			if (pageNumber.equals("1"))
			{	//validate name
				String name=(String)wizardStateRead.getFirstValue("name");
				if (name.length()==0)
				{	validation.addNode("name", "Name must be specified");
				}
				
				//validate urn
				String urn=(String)wizardStateRead.getFirstValue("urn");
				List<IModule> modules=ModuleManager.getSingleton().getModules();
				IModule found=null;
				for (IModule m: modules)
				{	if (m.getMeta().getIdentifier().equals(urn))
					{	found=m;
						break;
					}
				}
				if (found!=null)
				{	validation.addNode("urn", "Module with this identifier already exists");
				}
				else if (urn.length()<=4)
				{	validation.addNode("urn", "Identifier must be specified");
				}
				
				//validate version
				try
				{	Version v=new Version((String)wizardStateRead.getFirstValue("version"));
				}
				catch (Exception e)
				{	validation.addNode("version", "Malformed version specification");
				}
			}
			
			if (pageNumber.equals("2"))
			{	//validate source
				String source=(String)wizardStateRead.getFirstValue("source");
				File sourceDir=new File(source);
				if (sourceDir.exists())
				{	validation.addNode("source", "Source directory already exists");
				}
				else
				{	File parent=sourceDir.getParentFile();
					//System.out.println(parent);
					if (!parent.exists())
					{	validation.addNode("source", "Parent directory ["+parent.toString()+"] does not exist");
					}
				}
				String external = (String)wizardStateRead.getFirstValue("external");
				if (!external.startsWith("/") || !external.endsWith("/"))
				{	validation.addNode("external", "Must specify an absolute base directory path");
				}
				String internal = (String)wizardStateRead.getFirstValue("internal");
				if (!internal.startsWith("/") || !internal.endsWith("/"))
				{	validation.addNode("internal", "Must specify an absolute base directory path");
				}
			}
			
			if (pageNumber.equals("3"))
			{	createNewModule(aContext,wizardStateRead);
			}
			
		}
		
		//System.out.println(validation.getRoot());
		
		if (submit && validation.getRoot().getValues("/*/*").length==0)
		{	//passed validation
			if (pageNumber.equals("1"))
			{	pageNumber="2";
				String base=aContext.getKernelContext().getKernel().getConfiguration().getString("netkernel.install.path");
				String name=(String)wizardStateRead.getFirstValue("urn");
				StringBuilder sb = new StringBuilder(name.length());
				for (int i=0; i<name.length(); i++)
				{	char c = name.charAt(i);
					if (Character.isLetterOrDigit(c))
					{	sb.append(c);
					}
					else if (c==':')
					{	sb.append('.');
					}
				}
				sb.append('/');
				String srcname=sb.toString();
				String dir=base+"modules/"+srcname;
				wizardState.setCursor(wizardStateRead.getFirstNode("source"));
				wizardState.setValue(new URI(base).resolve(dir).getSchemeSpecificPart());
				
				name=(String)wizardStateRead.getFirstValue("name");
				sb.setLength(0);
				for (int i=0; i<name.length(); i++)
				{	char c = name.charAt(i);
					if (Character.isLetterOrDigit(c))
					{	sb.append(c);
					}
					else
					{	sb.append('_');
					}
				}
				String ext=sb.toString();
				
				wizardState.setCursor(wizardStateRead.getFirstNode("external"));
				wizardState.setValue("/"+ext+"/");
				aContext.sink("session:/newModuleWizardState", wizardStateRead);
			}
			else if (pageNumber.equals("2"))
			{	pageNumber="3";
				aContext.sink("session:/newModuleWizardState", wizardStateRead);
			}
			else if (pageNumber.equals("3"))
			{	pageNumber="4";
				aContext.sink("session:/newModuleWizardState", wizardStateRead);
			}
			
			

			wizardState.setCursor(wizardStateRead.getFirstNode("page"));
			wizardState.setValue(pageNumber);
			
		}
		
		//System.out.println(wizardState.getRoot());
		
		INKFRequest req=aContext.createRequest("active:xslt");
		req.addArgument("operator", "res:/tools/deployment/styleNewModuleWizard.xsl");
		req.addArgumentByValue("operand", wizardState.getRoot());
		req.addArgumentByValue("validation",validation.getRoot());
		req.setRepresentationClass(aContext.getThisRequest().getRepresentationClass());
		INKFResponseReadOnly resp=aContext.issueRequestForResponse(req);
		INKFResponse resp2=aContext.createResponseFrom( resp );
	    resp2.setHeader("WrappedControlPanel", true);
	    resp2.setHeader("WrappedControlPanelSubtitle", "page "+pageNumber+" of  4");
		
	}
	
	
	private void createNewModule(INKFRequestContext aContext, IHDSNode wizardStateRead) throws Exception
	{
		//create module source directory
		File sourceDir = new File((String)wizardStateRead.getFirstValue("source"));
		sourceDir.mkdir();
		String sourceIdentifer=sourceDir.toURI().toString();
		
		try
		{
		
			IXDA moduleXML=aContext.source("res:/tools/deployment/stub/module.xml",IXDAReadOnly.class).getClone();
			
			//setup basic details in module.xml
			String urn= (String)wizardStateRead.getFirstValue("urn");
			moduleXML.setText("/module/meta/identity/uri", urn);
			moduleXML.setText("/module/meta/identity/version",  (String)wizardStateRead.getFirstValue("version"));
			String name=(String)wizardStateRead.getFirstValue("name");
			moduleXML.setText("/module/meta/info/name",  name);
			String desc=(String)wizardStateRead.getFirstValue("desc");
			moduleXML.setText("/module/meta/info/description",  desc);
			
			//setup icon
			String internal = (String)wizardStateRead.getFirstValue("internal");
			String external = (String)wizardStateRead.getFirstValue("external");
			File pubDir = new File(sourceDir,internal+"pub");
			pubDir.mkdirs();
			copyResource(aContext,"res:/tools/deployment/stub/icon.png",pubDir.toURI().toString()+"/icon.png");
			moduleXML.setText("/module/meta/info/icon",  "res:"+external+"pub/icon.png");
			moduleXML.setText("/module/rootspace[1]/fileset[1]/regex", "res:"+external+"pub/(.*)");
			moduleXML.setText("/module/rootspace[1]/fileset[1]/rewrite", "res:"+internal+"pub/$1");
			
			//setup example service
			String example=(String)wizardStateRead.getFirstValue("example");
			if (example.equals("dpml"))
			{	// copy dpml mapper code into module.xml
				IXDA dpml = aContext.source("res:/tools/deployment/stub/dpml_example.xml",IXDAReadOnly.class).getClone();
				dpml.setText("/mapper/config/endpoint[1]/id", urn+":ExampleEndpoint");
				dpml.setText("/mapper/config/endpoint[1]/name", "Example Service");
				dpml.setText("/mapper/config/endpoint[1]/description", "Example DPML service generated by New Module Wizard");
				dpml.setText("/mapper/config/endpoint[1]/grammar/group[1]", external+"service");
				dpml.setText("/mapper/config/endpoint[1]/request/argument[1]", "res:"+internal+"endpoints/example.dpml");
				dpml.setText("/mapper/space/fileset/regex", "res:"+internal+"endpoints/.*");
				moduleXML.append(dpml, "/", "/module/rootspace[1]");
				
				//copy dpml script
				File endpointDir = new File(sourceDir,internal+"endpoints");
				endpointDir.mkdirs();
				copyResource(aContext,"res:/tools/deployment/stub/example.dpml",endpointDir.toURI().toString()+"/example.dpml");
			}
			else if (example.equals("groovy"))
			{	// copy groovy mapper code into module.xml
				IXDA groovy = aContext.source("res:/tools/deployment/stub/groovy_example.xml",IXDAReadOnly.class).getClone();
				groovy.setText("/mapper/config/endpoint[1]/id", urn+":ExampleEndpoint");
				groovy.setText("/mapper/config/endpoint[1]/name", "Example Service");
				groovy.setText("/mapper/config/endpoint[1]/description", "Example Groovy service generated by New Module Wizard");
				groovy.setText("/mapper/config/endpoint[1]/grammar/group[1]", external+"service");
				groovy.setText("/mapper/config/endpoint[1]/request/argument[1]", "res:"+internal+"endpoints/example.gy");
				groovy.setText("/mapper/space/fileset/regex", "res:"+internal+"endpoints/.*");
				moduleXML.append(groovy, "/", "/module/rootspace[1]");
				
				//copy dpml script
				File endpointDir = new File(sourceDir,internal+"endpoints");
				endpointDir.mkdirs();
				copyResource(aContext,"res:/tools/deployment/stub/example.gy",endpointDir.toURI().toString()+"/example.gy");
				
			}
			
			//setup dynamic import to fulcrum
			File etcDir = new File(sourceDir,"etc/system");
			etcDir.mkdirs();
			String fulcrum=(String)wizardStateRead.getFirstValue("fulcrum");
			if (fulcrum.length()>0)
			{	String doc = "<connection><type>"+fulcrum+"</type></connection>";
				aContext.sink(etcDir.toURI().toString()+"/SimpleDynamicImportHook.xml",doc);
			}
			
			//setup documentation
			if (wizardStateRead.getFirstValue("doc").equals("on"))
			{	//copy and fill Docs.xml
				IXDA docs = aContext.source("res:/tools/deployment/stub/Docs.xml",IXDAReadOnly.class).getClone();
				docs.setText("/docs/doc[1]/id", urn+":guide");
				docs.setText("/docs/doc[1]/title","Guide to "+name);
				docs.setText("/docs/doc[1]/uri","res:"+external+"pub/doc_guide.txt");

				docs.setText("/docs/doc[2]/id", urn+":example");
				docs.setText("/docs/doc[2]/title","Reference for example endpoint");
				docs.setText("/docs/doc[2]/uri","res:"+external+"pub/doc_example.txt");
				aContext.sink(sourceIdentifer+"/etc/system/Docs.xml", docs);
				
				//copy and fill Books.xml
				IXDA books = aContext.source("res:/tools/deployment/stub/Books.xml",IXDAReadOnly.class).getClone();
				books.setText("/books/book/id", "book:"+urn);
				books.setText("/books/book/title", name);
				books.setText("/books/book/desc", desc);
				books.setText("/books/book/toc/item[1]/@id", urn+":guide");
				books.setText("/books/book/toc/item[2]/@id", urn+":example");
				aContext.sink(sourceIdentifer+"/etc/system/Books.xml",books);
				
				// copy guide stub
				copyResource(aContext,"res:/tools/deployment/stub/guide.txt",pubDir.toURI().toString()+"/doc_guide.txt");
				
				//generate example document
				StringBuilder docBody = new StringBuilder();
				docBody.append("{endpoint}"+urn+":ExampleEndpoint{/endpoint}\n");
				docBody.append("==Detail==\n");
				docBody.append("This example endpoint simply formats the value argument it receives and returns it as a string response. Logging it along the way.\n");
				docBody.append("\n");
				docBody.append("==Resource Identifier Syntax==\n");
				docBody.append("The endpoint accepts requests which match the identifier syntax of '''res:"+external+"service/ANY_OTHER_TEXT'''\n");
				docBody.append("\n");
				docBody.append("The '''ANY_OTHER_TEXT''' is captured as the value argument passed into the endpoint.\n");
				
				aContext.sink(pubDir.toURI().toString()+"/doc_example.txt",docBody.toString());
			}
			
			//setup unit test
			if (wizardStateRead.getFirstValue("test").equals("on"))
			{	//fill and append test rootspace
				IXDA testRootspace = aContext.source("res:/tools/deployment/stub/test_rootspace.xml",IXDAReadOnly.class).getClone();
				testRootspace.setText("/rootspace/@name", "Unit Test Space");
				testRootspace.setText("/rootspace/fileset[2]/regex", "res:"+internal+"tests/.*");
				testRootspace.setText("/rootspace/import/uri", urn);
				moduleXML.append(testRootspace, "/", "/module/");
				
				//copy and fill /etc/system/Tests.xml
				IXDA tests = aContext.source("res:/tools/deployment/stub/Tests.xml",IXDAReadOnly.class).getClone();
				tests.setText("/tests/test/id", "test:"+urn);
				tests.setText("/tests/test/name", name);
				tests.setText("/tests/test/desc", desc);
				tests.setText("/tests/test/uri", "res:"+internal+"tests/testlist.xml");
				aContext.sink(sourceIdentifer+"/etc/test/Tests.xml", tests);
				
				//copy and fill testlist.xml
				File testsDir = new File(sourceDir,internal+"tests");
				testsDir.mkdirs();
				IXDA testlist = aContext.source("res:/tools/deployment/stub/testlist.xml",IXDAReadOnly.class).getClone();
				testlist.setText("/testlist/test[1]/request/identifier", "res:"+external+"service/Hello");
				testlist.setText("/testlist/test[2]/request/identifier", "meta:ExampleEndpoint");
				aContext.sink(testsDir.toURI().toString()+"/testlist.xml",testlist);
			}
			
			aContext.sink(sourceIdentifer+"/module.xml", tidyIndentXML(moduleXML,aContext));
			
			//update modules.xml
			IConfiguration config=aContext.getKernelContext().getKernel().getConfiguration();
			String installPathS=config.getString("netkernel.install.path");
			String initModulesS=config.getString("netkernel.init.modules");
			URI installPath=URI.create(installPathS);
			URI initModules=installPath.resolve(initModulesS);
			IXDA modulesXML = aContext.source(initModules.toString(),IXDAReadOnly.class).getClone();
			modulesXML.appendPath("/modules", "module", sourceIdentifer);
			aContext.sink(initModules.toString(),modulesXML);
			
		}
		catch (Exception e)
		{	cleanupDir(sourceDir);
			throw e;
		}
		
	}
	
	private void copyResource(INKFRequestContext aContext, String aSrc, String aDest) throws Exception
	{	Object representation=aContext.source(aSrc);
		aContext.sink(aDest, representation);
	}
	
	private void appendComment(IXDA aXDA, String aPath, String aComment) throws XPathLocationException
	{	DOMXDA xda = (DOMXDA)aXDA;
		Node commentNode = xda.getRoot().getOwnerDocument().createComment(aComment);
		xda.getSingleNode(aPath).appendChild(commentNode);
	}
	
	private Object tidyIndentXML(Object aIn, INKFRequestContext aContext) throws Exception
	{	
		INKFRequest req=aContext.createRequest("active:XMLTidy");
		req.addArgumentByValue("operand", aIn);
		Object rep=aContext.issueRequest(req);
		return rep;
		
	}
	
	public static void cleanupDir(File aDir)
	{	File[] files=aDir.listFiles();
		for (File f: files)
		{	if (f.isDirectory())
			{	cleanupDir(f);
				f.delete();
			}
			else
			{	f.delete();
			}
		}
		aDir.delete();
	}

}
