package tools.explorer;

import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.netkernel.ext.system.representation.*;
import org.netkernel.ext.system.transrepresentation.MetaToXML;
import org.netkernel.layer0.boot.IModule;
import org.netkernel.layer0.meta.*;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.util.*;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.urii.*;
import org.netkernel.xml.xda.DOMXDA;
import org.netkernel.xml.xda.IXDAIterator;
import org.w3c.dom.*;

public class DetailPanelAccessor extends StandardAccessorImpl
{
	public void onSource(INKFRequestContext context) throws Exception
	{
		
		INKFRequestReadOnly reqIn=context.getThisRequest();
		String hash=reqIn.getArgumentValue("hash");


		Object rep=null;

		if (hash!=null)
		{	//generate detail panel

			String moduleHash=reqIn.getArgumentValue("module");
			String spaceHash=reqIn.getArgumentValue("space");
			String elementHash=reqIn.getArgumentValue("element");
			IRepDeployedModules deployedModules=(IRepDeployedModules)context.source("active:moduleList",IRepDeployedModules.class);
			String resourceSpaceHash=getResourceSpaceHash(moduleHash, spaceHash,deployedModules);
			IMetaRepresentation meta = deployedModules.metadataForHash(hash);
			if (meta==null)
			{	throw new NKFException("Explorer entity not found",hash);
			}
			
			int k=hash.lastIndexOf('/');
			String parentHash=(k>=0)?hash.substring(0,k):"";
			Document metaDoc=(Document)context.transrept(meta, Document.class);  
			
			context.setCWU(context.getRuntimeSourceIdentifier());
			INKFRequest req = context.createRequest("active:xslt");
			req.addArgument("operator","res:/tools/explorer/styleMeta2.xsl");
			req.addArgumentByValue("resourceSpaceHash", resourceSpaceHash);
			req.addArgumentByValue("parentHash", parentHash);
			req.addArgumentByValue("operand",metaDoc);
			req.addArgument("deployed","active:moduleListDoc");
			
			// append any documentation
			IIdentifier id = meta.getHumanReadableDocumentation();
			if (id!=null && resourceSpaceHash!=null)
			{	
				try
				{
					INKFRequest req2 = context.createRequest(id.toString());
					ISpace resourceSpace = (ISpace)deployedModules.componentForHash(resourceSpaceHash);
					
					IRequestScopeLevel hrdRctx = RequestScopeLevelImpl.createNonDurableScopeLevel(resourceSpace,context.getKernelContext().getThisKernelRequest().getRequestScope());
					req2.setRequestScope(hrdRctx);
					req2.setRepresentationClass(String.class);
					INKFResponseReadOnly hrdResp=context.issueRequestForResponse(req2);
					String hrdString;
					
					INKFRequest docReq=context.createRequest("active:wikiParser/XHTML");
					docReq.addArgumentByValue("operand",hrdResp.getRepresentation());
					docReq.setRepresentationClass(String.class);
					docReq.addArgument("base","/");
					hrdString=(String)context.issueRequest(docReq);
					
					//Do the book cross linking (sync)
					Map bookmap=(Map)context.source("active:documentBookMap");
					Pattern mPattern=Pattern.compile("./wiki/(.*?)\\\"");
					Matcher m=mPattern.matcher(hrdString);
					int i=1;
					int start=0;
					while(m.find(start))
					{	String doc2=m.group(1);
						String sub=null;
						hash=deployedModules.hashForSpaceElement(doc2);
						if(hash!=null)
						{	sub="/tools/explore2/"+hash+"\"";
						}
						else
						{	String lbookid=(String)bookmap.get(doc2);
							if(lbookid==null)
							{	sub="./"+doc2+"\"";
							}
							else
							{	sub="../"+lbookid+"/"+doc2+"\"";;		
							}
						}
						hrdString=m.replaceFirst(sub);
						start=m.start();
						m.reset(hrdString);
					}
					req.addArgumentByValue("hrd",hrdString);
				}
				catch (Exception e)
				{	// failed to find doc
					System.out.println(Utils.throwableToString(e));
				}
			}
			
			//append javadoc if necessary
			if (meta instanceof IRepresentationMeta)
			{	IRepresentationMeta rm = (IRepresentationMeta)meta;
				String implName=rm.getImplementation().getName();
				
				INKFRequest req2=context.createRequest("active:nkseSearch");
				String search="title:("+implName+") AND category:javadoc";
				req2.addArgumentByValue("operand", search);
				req2.setRepresentationClass(IHDSNode.class);
				IHDSNode sr = (IHDSNode)context.issueRequest(req2);
				IHDSNode firstHit=sr.getFirstNode("/hits/hit");
				if (firstHit!=null)
				{	float score=Float.parseFloat(firstHit.getFirstValue("score").toString());
					if (score>1.0f)
					{	String documentID=null;
						String resolverID=null;
						for(IHDSNode field: firstHit.getNodes("field"))
						{	String name=(String)field.getFirstValue("name");
							if(name.equals("documentID"))
							{	documentID=(String)field.getFirstValue("value");
							}
							else if(name.equals("resolverID"))
							{	resolverID=(String)field.getFirstValue("value");
							}
						}
						if (documentID!=null && resolverID!=null)
						{
							req2=context.createRequest("active:nkseSearchResolver");
							req2.addArgumentByValue("documentID", documentID);
							req2.addArgumentByValue("resolverID", resolverID);
							req2.setRepresentationClass(String.class);
							String path=(String)context.issueRequest(req2);
							//System.out.println("path="+path);
							
							INKFResponseReadOnly resp=context.sourceForResponse("res:"+path);
							req2=context.createRequest("active:tagSoup");
							req2.addArgumentFromResponse("operand", resp);
							req2.setRepresentationClass(Document.class);
							Document d = (Document)context.issueRequest(req2);
							DOMXDA xda = new DOMXDA(d,true);
							xda.removeNS("/html", "");
							int j=path.lastIndexOf("/");
							String javadocBase=path.substring(0,j+1);
							for (IXDAIterator i=xda.iterator("//a"); i.hasNext(); )
							{	i.next();
								try
								{	String href=i.getText("@href", false);
									i.setText("@href", javadocBase+href);
								}
								catch (Exception e)
								{
								}
							}
							
							req.addArgumentByValue("javadoc", xda.getDocument());
							
						}
					}
				}
			}
			
			

			rep=context.issueRequest(req);
		}
		else
		{	context.setCWU(context.getRuntimeSourceIdentifier());
			rep=context.source("res:/tools/explorer/title.xml", Document.class);	
		}

		context.createResponseFrom(rep).setExpiry(INKFResponse.EXPIRY_ALWAYS);
		
	}
	
	private String getResourceSpaceHash(String aModule, String aSpace, IRepDeployedModules aDeployedModules)
	{
		String result;
		if (aSpace!=null)
		{	result=aModule+"/"+aSpace;
		}
		else
		{	// find first public space in module
			result="none";
			IModule module=(IModule)aDeployedModules.componentForHash(aModule);
			if (module!=null)
			{	ISpace[] spaces = module.getPublicSpaces();
				if (spaces.length>0)
				{	result = aDeployedModules.hashForComponent(spaces[0]);
				}
			}
		}
		return result;
	}
	
	
}
