<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xalan="http://xml.apache.org/xalan"
                   exclude-result-prefixes="xalan" xmlns:nk="http://1060.org">
	<xsl:import href="/nkse/style/xsl/xmlverbatim.xsl"/>
    <xsl:output method="xml"/>
    <xsl:param name="resourceSpaceHash" nk:class="java.lang.String"/>
    <xsl:param name="parentHash" nk:class="java.lang.String"/>
	
	<xsl:param name="deployed"><modules/></xsl:param>
	<xsl:param name="hrd"></xsl:param>
	<xsl:param name="javadoc"></xsl:param>
	
	<xsl:variable name="base">/tools/explore/</xsl:variable>
	<xsl:variable name="cr">/introspect/cr</xsl:variable>
	 
	<xsl:template match="/meta">

		<div class="meta-box" >

			<table>
				<tr>
					<td>
						<img>
						<xsl:choose>
							<xsl:when test="string-length(icon)">
								<xsl:attribute name="src"><xsl:value-of select="$cr"/>/<xsl:value-of select="$resourceSpaceHash"/>/<xsl:value-of select="icon"/></xsl:attribute>
							</xsl:when>
							<xsl:when test="contains(class,'IRepresentationMeta')">
								<xsl:attribute name="src">/tools/introspect/pub/representation.png</xsl:attribute>
							</xsl:when>
							<xsl:when test="contains(class,'IPrototypeMeta')">
								<xsl:attribute name="src">/tools/introspect/pub/prototype.png</xsl:attribute>
							</xsl:when>
							<xsl:when test="contains(class,'IEndpointMeta')">
								<xsl:attribute name="src">/tools/introspect/pub/endpoint.png</xsl:attribute>
							</xsl:when>
							<xsl:when test="contains(class,'ISpaceMeta')">
								<xsl:attribute name="src">/tools/introspect/pub/sphere.png</xsl:attribute>
							</xsl:when>
							<xsl:when test="contains(class,'IModuleMeta')">
								<xsl:attribute name="src">/tools/introspect/pub/module.png</xsl:attribute>
							</xsl:when>
							<xsl:when test="contains(class,'IPhysicalEndpointMeta')">
								<xsl:attribute name="src">/tools/introspect/pub/endpoint.png</xsl:attribute>
							</xsl:when>
						</xsl:choose>
						</img>
					</td>
					<td>
						<div style="font-weight: bold; font-size: 150%">
							<xsl:value-of select="name"/>
						</div>
						<div><xsl:value-of select="desc"/></div>
					</td>
				</tr>
			</table>

			<table>
			
			<xsl:choose>
				<xsl:when test="contains(class,'IEndpointMeta')">
					<xsl:call-template name="endpoint"/>
				</xsl:when>
				<xsl:when test="contains(class,'IPrototypeMeta')">
					<xsl:call-template name="prototype"/>
				</xsl:when>
				<xsl:when test="contains(class,'IRepresentationMeta')">
					<xsl:call-template name="representation"/>
				</xsl:when>
				<xsl:when test="contains(class,'ISpaceMeta')">
					<xsl:call-template name="space"/>
				</xsl:when>
				<xsl:when test="contains(class,'IModuleMeta')">
					<xsl:call-template name="module"/>
				</xsl:when>
				<xsl:when test="contains(class,'IPhysicalEndpointMeta')">
					<xsl:call-template name="physical"/>
				</xsl:when>
			</xsl:choose>
			
			<tr><td class="label">fields:</td>
			<td>
				<xsl:choose>
					<xsl:when test="fields/field">
						<table class="data-table">
							<tr class="vis-title"><td>Name</td><td>Value</td></tr>
							<xsl:for-each select="fields/field">
								<tr>
									<xsl:if test="@name='problem'">
										<xsl:attribute name="class">error</xsl:attribute>
									</xsl:if>
									<td><xsl:value-of select="@name"/></td>
									<td>
										<pre style="margin: 0px;"><xsl:value-of select="."/></pre>
									</td>
								</tr>
							</xsl:for-each>
						</table>
					</xsl:when>
					<xsl:otherwise>[none]</xsl:otherwise>
				</xsl:choose>				
			</td></tr>
			
			<xsl:if test="string-length($parentHash)">
				<tr><td class="label">parent:</td>
				<td>
					<a href="{$base}{$parentHash}">
						<xsl:value-of select="$deployed/modules/module[hash=$parentHash]/name"/>
						<xsl:value-of select="$deployed/modules/module/spaces/space[hash=$parentHash]/name"/>
					</a>
				</td>
				</tr>
			</xsl:if>
			
			<tr>
			<td  class="label">references:</td><td><a class="btn" href="/tools/search/?search={name}">search</a></td>
			</tr> 

			<xsl:if test="$hrd">
				<tr><td colspan="2"><xsl:copy-of select="$hrd"/></td>
				</tr>
			</xsl:if>	
			
			
			</table>	
				
		</div>
	</xsl:template>	
	
	<xsl:template name="module">
		<tr><td class="label">type:</td><td>module</td></tr>
		<tr><td class="label">identifier:</td><td><xsl:value-of select="identifier"/></td></tr>
		<tr><td class="label">version:</td><td><xsl:value-of select="version"/></td></tr>
		<tr><td class="label">source:</td><td><xsl:value-of select="source"/></td></tr>
		<tr><td class="label">spaces:</td><td>
			<table class="data-table">
			<tr class="vis-title"><td>Space</td><td>URI</td><td>Public</td></tr>
			<xsl:for-each select="spaces/space">
				<xsl:call-template name="module-space"/>
			</xsl:for-each>
			</table>
		</td></tr>
	</xsl:template>
	
	<xsl:template name="module-space">
		<xsl:param name="depth">0</xsl:param>
		<tr>
			<td style="padding-left: {$depth}em;"><a href="{$base}{hash}">
				<xsl:if test="problem">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
				<xsl:choose>
					<xsl:when test="string-length(name)"><xsl:value-of select="name"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="id"/></xsl:otherwise>
				</xsl:choose>
				</a></td>
			<td><xsl:value-of select="id"/></td>
			<td><xsl:value-of select="public"/></td>
		</tr>
		<xsl:for-each select="spaces/space">
			<xsl:call-template name="module-space">
				<xsl:with-param name="depth"><xsl:value-of select="$depth + 1"/></xsl:with-param>
			</xsl:call-template>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template name="space">
		<tr><td class="label">type:</td><td>space</td></tr>
		<tr><td class="label">public:</td><td><xsl:value-of select="public"/></td></tr>
		<tr><td class="label">identifier:</td><td>
			<xsl:choose>
				<xsl:when test="string-length(identifier)">
					<xsl:value-of select="identifier"/>
				</xsl:when>
				<xsl:otherwise>[none] (private)
				</xsl:otherwise>
			</xsl:choose>
		</td></tr>
		<tr><td class="label">version:</td><td>
			<xsl:choose>
				<xsl:when test="string-length(version)">
					<xsl:value-of select="version"/>
				</xsl:when>
				<xsl:otherwise>[none] (private)
				</xsl:otherwise>
			</xsl:choose>
		</td></tr>
		 
		<tr><td class="label">mappings:</td><td>
			<table class="data-table" style="font-size: 8pt;">
			<tr class="vis-title"><td>Mapping</td><td>URI</td><td>Type</td><td>Delegate space</td></tr>
			<xsl:for-each select="endpoints/endpoint">
				<tr class="vis-title">
					<td colspan="5">
						<div>
						<xsl:if test="problem">
							<xsl:attribute name="class">error</xsl:attribute>
						</xsl:if>
						<a href="{$base}{hash}"><xsl:value-of select="name"/></a>
						</div>
					</td>
				</tr>
				<xsl:for-each select="element">
					<tr>
						<xsl:if test="problem">
							<xsl:attribute name="class">error</xsl:attribute>
						</xsl:if>
						<td><a href="{$base}{hash}">
							<xsl:call-template name="wrapped">
								<xsl:with-param name="text">
									<xsl:choose>
										<xsl:when test="string-length(name)"><xsl:value-of select="name"/></xsl:when>
										<xsl:otherwise><xsl:value-of select="id"/></xsl:otherwise>
									</xsl:choose>
								</xsl:with-param>
							</xsl:call-template>
						</a></td>
						<!-- <td><xsl:value-of select="desc"/></td> -->
						<td>
							<xsl:call-template name="wrapped">
								<xsl:with-param name="text"><xsl:value-of select="id"/></xsl:with-param>
							</xsl:call-template>						
						</td>
						<td><xsl:value-of select="type"/></td>
						<td>
							<xsl:choose>
								<xsl:when test="local='true'">
									local
								</xsl:when>
								<xsl:otherwise>
									<a href="{$base}{delegate-hash}"><xsl:value-of select="delegate-name"/></a>
								</xsl:otherwise>
							</xsl:choose>
						</td>							
					</tr>
				</xsl:for-each>
			</xsl:for-each>
			</table>
		</td></tr>
	</xsl:template>

	<xsl:template name="physical">
		<tr><td class="label">type:</td><td>physical endpoint</td></tr>
		<tr><td class="label">implementation:</td><td><xsl:value-of select="implementation"/></td></tr>
		<tr><td class="label">elements:</td><td>
			<table class="data-table">
				<tr class="vis-title"><td>Mapping</td><td>URI</td><td>Type</td><td>delegate space</td></tr>
				<xsl:for-each select="elements/element">
					<tr>
						<xsl:if test="problem">
							<xsl:attribute name="class">error</xsl:attribute>
						</xsl:if>
						<td><a href="{$base}{hash}">
							<xsl:call-template name="wrapped">
								<xsl:with-param name="text"><xsl:value-of select="name"/></xsl:with-param>
							</xsl:call-template>						
							</a></td>
						<td>
							<xsl:call-template name="wrapped">
								<xsl:with-param name="text"><xsl:value-of select="id"/></xsl:with-param>
							</xsl:call-template>						
						</td>
						<td><xsl:value-of select="type"/></td>
						<td>
							<xsl:choose>
								<xsl:when test="local='true'">
									local
								</xsl:when>
								<xsl:otherwise>
									<a href="{$base}{delegate-hash}"><xsl:value-of select="delegate-name"/></a>
								</xsl:otherwise>
							</xsl:choose>
						</td>
					</tr>
				</xsl:for-each>
				</table>
		</td></tr>
	</xsl:template>
			
	<xsl:template name="endpoint">
		<tr><td class="label">type:</td><td>logical endpoint</td></tr>
		<tr><td class="label">verbs:</td><td>
			<xsl:choose>
				<xsl:when test="interface/verbs/verb">
					<xsl:for-each select="interface/verbs/verb">
						<xsl:value-of select="."/>
						<xsl:if test="position() != last()">, </xsl:if>
					</xsl:for-each>
				</xsl:when>
				<xsl:otherwise>[none]</xsl:otherwise>
			</xsl:choose>
		</td></tr>
		<tr><td class="label">grammar:</td><td>
			<!--<xsl:value-of select="interface/grammar"/>-->
			<xsl:for-each select="interface/grammar">
				<xsl:call-template name="top2"/>			
			</xsl:for-each>
		</td></tr>
		<tr><td class="label">arguments:</td><td>
			<xsl:choose>
				<xsl:when test="interface/args/arg">
					<table>
					<tr class="table-header"><td>Name</td><td>isIdentifier?</td><td>Representation</td></tr>
						<xsl:for-each select="interface/args/arg">
						<tr>
							<td><xsl:value-of select="name"/></td>
							<td><xsl:value-of select="isId"/></td>
							<td>
								<xsl:for-each select="reps/rep">
									<xsl:choose>
										<xsl:when test="hash">
											<a href="{$base}{hash}"><xsl:value-of select="name"/></a>
										</xsl:when>
										<xsl:otherwise><xsl:value-of select="class"/></xsl:otherwise>
									</xsl:choose>
									<xsl:if test="position() != last()">, </xsl:if>
								</xsl:for-each>								
							</td>
						</tr>
					</xsl:for-each>
					</table>					
				</xsl:when>
				<xsl:otherwise>[none]</xsl:otherwise>
			</xsl:choose>
		</td></tr>
		<tr><td>returns:</td><td>
			<xsl:choose>
				<xsl:when test="interface/reps/rep">
					<xsl:for-each select="interface/reps/rep">
						<!--<xsl:message><xsl:value-of select="hash"/></xsl:message>-->
						<xsl:choose>
							<xsl:when test="hash">
								<a href="{$base}{hash}"><xsl:value-of select="name"/></a>
							</xsl:when>
							<xsl:otherwise><xsl:value-of select="class"/></xsl:otherwise>
						</xsl:choose>
						
						<xsl:if test="position() != last()">, </xsl:if>
					</xsl:for-each>								
				</xsl:when>
				<xsl:otherwise>[none]</xsl:otherwise>
			</xsl:choose>
		</td></tr>
		<tr><td>exceptions:</td><td>
			<xsl:choose>
				<xsl:when test="interface/exs/ex">
					<table>
					<tr class="table-header"><td>Id</td><td>Description</td></tr>
					<xsl:for-each select="interface/exs/ex">
						<tr>
							<td><xsl:value-of select="name"/></td>
							<td><xsl:value-of select="desc"/></td>
						</tr>
					</xsl:for-each>
					</table>
				</xsl:when>
				<xsl:otherwise>[none]</xsl:otherwise>
			</xsl:choose>
		</td></tr>
			
	</xsl:template>
			
	<xsl:template name="prototype">
		<tr><td class="label">type:</td><td>prototype</td></tr>
		<tr><td class="label">identifier:</td><td><xsl:value-of select="identifier"/></td></tr>
		<tr><td class="label">class:</td><td><xsl:value-of select="implementation"/></td></tr>
		<tr><td class="label">parameters:</td><td>
			<table>
				<tr class="table-header"><td>Name</td><td>Description</td><td>Class</td><td>Default</td></tr>
			<xsl:for-each select="parameters/parameter">
				<tr>
					<td><xsl:value-of select="name"/></td>
					<td><xsl:value-of select="desc"/></td>
					<td><xsl:value-of select="class"/></td>
					<td><xsl:value-of select="default"/></td>
				</tr>
			</xsl:for-each>			
			</table>
		</td></tr>
	</xsl:template>
	
	<xsl:template name="representation">
		<xsl:choose>
			<xsl:when test="$javadoc">
				<tr><td colspan="2">
					<xsl:copy-of select="$javadoc/*/*[3]/*"/>
				</td></tr>
			</xsl:when>
			<xsl:otherwise>
				<tr><td class="label">type:</td><td>representation</td></tr>
				<tr><td class="label">implementation:</td><td><xsl:value-of select="implementation"/></td></tr>
				<tr><td class="label">interface:</td><td>
					<div class="error">JavaDoc not found for this representation. <a href="/tools/javadoc/">Generate</a></div>
					<xsl:for-each select="methods/method">
						<div>
							<xsl:value-of select="return"/>&#160;<xsl:value-of select="name"/>
							(
							<xsl:for-each select="params/param"><xsl:value-of select="."/>
								<xsl:if test="position() != last()">, </xsl:if>
							</xsl:for-each>
							)
							<xsl:if test="exceptions/exception">throws
								<xsl:for-each select="exceptions/exception">
									<xsl:value-of select="."/>
									<xsl:if test="position() != last()">, </xsl:if>
								</xsl:for-each></xsl:if>;
						</div>
					</xsl:for-each>
					</td></tr>
				</xsl:otherwise>
			</xsl:choose>
	</xsl:template>
	
		<xsl:template name="wrapped">
		<xsl:param name="text"/>
		<xsl:choose>
			<xsl:when test="string-length($text)>30">
				<xsl:value-of select="substring($text,1,30)"/><wbr/><xsl:call-template name="wrapped">
					<xsl:with-param name="text" select="substring($text,30+1)"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$text"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
</xsl:stylesheet>